import { IntervalSorted, PointContainment } from '../index';
/**
 * A number range between two values ([[T0]] & [[T1]]).
 * This range is increasing when [[T0]] < [[T1]] and decreasing when [[T0]] > [[T1]].
 *
 * ### Example
 * ```js
 * import { Interval } from 'shapetypes'
 *
 * // Create an interval
 * const interval = new Interval(5, 10);
 *
 * // Get properties of the interval
 * console.log(interval.length);
 * // => 5
 * console.log(interval.mid);
 * // => 7.5
 * console.log(interval.contains(8));
 * // => True
 * console.log(interval.isIncreasing);
 * // => True
 *
 * // Create the same interval but in reverse
 * const reversed = new Interval(10, 5);
 *
 * // Get properties of the reversed interval
 * console.log(reversed.length);
 * // => -5
 * console.log(reversed.contains(8));
 * // => True
 * console.log(reversed.isIncreasing);
 * // => False
 * ```
 *
 */
export declare class Interval {
    /***
     * Creates an interval that encompasses a set of values.
     *
     * ### Example
     * ```js
     * const interval = Interval.fromValues([5, 3, 4]);
     * console.log(interval.min);
     * // => 3
     * console.log(interval.max);
     * // => 5
     * ```
     *
     * @category Create
     * @param values  Values to contain within the interval.
     */
    static fromValues(values: readonly number[]): Interval;
    /***
     * Creates an interval that encompasses two intervals.
     *
     * @category Create
     * @param a   First interval to encompass.
     * @param b   Second interval to encompass.
     */
    static union(a: Interval, b: Interval): Interval;
    /***
     * Finds the overlapping portion of two intervals and returns the resulting interval.
     *
     * @category Create
     * @param a     First interval to intersect
     * @param b   Second interval to intersect
     * @returns   An interval representing the overlap between these two intervals. If there is no overlap, returns undefined.
     */
    static intersection(a: Interval, b: Interval): Interval | undefined;
    private readonly _T0;
    private readonly _T1;
    /***
     * Creates an interval from two values.
     * @param T0  The start of the interval. Could either be the interval's min or max. If the interval is increasing, this should be the min.
     * @param T1  The end of the interval. Could either be the interval's min or max value. If the interval is increasing, this should be the max.
     */
    constructor(T0: number, T1: number);
    /**
     * Gets the start of the interval.
     */
    get T0(): number;
    /**
     * Gets the end of the interval.
     */
    get T1(): number;
    /**
     * Checks whether [[T0]] > [[T1]]. Returns true if it is.
     */
    get isDecreasing(): boolean;
    /**
     * Checks whether [[T0]] < [[T1]]. Returns true if it is.
     */
    get isIncreasing(): boolean;
    /**
     * Checks whether [[T0]] and [[T1]] are the same value. Returns true if they are.
     */
    get isSingleton(): boolean;
    /**
     * Gets the smaller of [[T0]] and [[T1]].
     */
    get min(): number;
    /**
     * Gets the value in the middle of [[T0]] and [[T1]].
     */
    get mid(): number;
    /**
     * Gets the larger of [[T0]] and [[T1]].
     */
    get max(): number;
    /**
     * Gets the signed distance between [[T0]] and [[T1]].
     * It will be positive if the interval is increasing and negative if it is decreasing.
     */
    get length(): number;
    /**
     * Gets the absolute distance between [[T0]] and [[T1]].
     * It will be positive regardless of whether the interval is increasing or decreasing.
     */
    get lengthAbs(): number;
    /**
     * Converts the interval into an [[IntervalSorted]] and returns the sorted interval.
     */
    asSorted(): IntervalSorted;
    /**
     * Checks whether a value is within the interval.
     * @param value     The value to check for containment.
     * @param strict    If true, the value must be entirely inside the interval and can't equal [[min]] or [[max]].
     *                  If false, the value can equal [[min]] or [[max]].
     * @param tolerance The amount the value can be outside the interval and still be considered inside.
     */
    contains(value: number, strict?: boolean, tolerance?: number): boolean;
    /**
     * Checks whether the value is within the interval and returns the result using
     * the [[PointContainment]] enum.
     * The value will be considered coincident if it equals the interval's min or max.
     * @param value       The value to check for containment.
     * @param tolerance   The distance the value can be from the interval's min or max and still considered coincident.
     */
    containsPoint(value: number, tolerance?: number): PointContainment;
    /***
     * Checks whether another interval has the same [[T0]] and [[T1]] values. Returns true if it does.
     * @param otherInterval  The interval to compare against.
     */
    equals(otherInterval: Interval): boolean;
    /**
     * Creates a copy of this interval expanded to contain a given value.
     * The direction of the interval, whether it is increasing or decreasing, will
     * be preserved. Depending on where the value falls in relation to the interval,
     * either T0 or T1 will change to accommodate the value.
     *
     * @param toInclude   Value to contain within the new interval.
     */
    grow(toInclude: number): Interval;
    /**
     * Remaps a value from the global number system into the normalized parameters of this interval.
     * See [[valueAt]] to understand how the parameters are calculated.
     *
     * @param value Value to remap to the parameter of the interval.
     * @returns   The value as a normalized parameter for the interval.
     */
    remapToInterval(value: number): number;
    /**
     * Creates a new interval equal to [-[[T1]], -[[T0]]].
     */
    reverse(): Interval;
    /**
     * Swaps [[T0]] and [[T1]] to create a new interval. The new interval is equal to [[[T1]], [[T0]]].
     */
    swap(): Interval;
    /***
     * Gets the interval as a string in the format: `[T0,T1]`.
     */
    toString(): string;
    /**
     * Remaps a value from normalized parameters of this interval into the global number system.
     * The interval's parameters range from 0 to 1.
     *
     * A parameter of 0 is equal to the start of the interval ([[T0]]). A parameter of
     * 0.5 is the mid point of the interval. And a parameter of 1 is the end of the interval ([[T1]]).
     *
     * @param t   The parameter to remap.
     * @returns   The parameter remapped to the global number system.
     */
    valueAt(t: number): number;
    /**
     * Creates a copy of the interval with a different T0 value.
     * @param newT0
     */
    withT0(newT0: number): Interval;
    /**
     * Creates a copy of the interval with a different T1 value.
     * @param newT1
     */
    withT1(newT1: number): Interval;
}
