import { Geometry, Point, Transform, Vector } from '../index';
/**
 * A 2d frame with a center point ([[origin]]) and two perpendicular axes ([[xAxis]] and [[yAxis]]).
 * The axes are always right-handed.
 *
 * ### Example
 * ```js
 * import { Plane, Point, Vector } from 'shapetypes';
 *
 * // Create a plane
 * const plane = new Plane(new Point(3,4), Vector.worldX());
 *
 * // Map a point from the global coordinate system to the plane's local coordinates
 * const worldPoint = new Point(5,6);
 * const planePoint = plane.remapToPlaneSpace(worldPoint);
 * console.log(planePoint.toString());
 * // => [2,2]
 *
 * // Remap the local point back to the global coordinate system
 * const output = plane.pointAt(planePoint);
 * console.log(output.toString());
 * // => [5,6]
 *
 * // Translate the plane
 * const moved = plane.translate(new Vector(10,20));
 * console.log(moved.origin.toString());
 * // => (13,24)
 *
 * ```
 */
export declare class Plane extends Geometry {
    /**
     * Returns the environment's base plane.
     * This plane has an [[origin]] of [0,0], an [[xAxis]] of [1,0], and a [[yAxis]] of [0,1].
     * @category Create
     */
    static worldXY(): Plane;
    /**
     * Creates a plane defined by a center point and a point on the x-axis.
     * @category Create
     * @param origin      The center of the plane.
     * @param axisPoint   A point on the plane's x-axis.
     */
    static fromPoints(origin: Point, axisPoint: Point): Plane;
    private static readonly _worldXY;
    private readonly _origin;
    private readonly _xAxis;
    private _cacheYAxis;
    private _cacheRemap;
    /***
     * Creates a plane from a center point and x-axis.
     * @param origin    The center of the plane.
     * @param xAxis     The direction of the [[xAxis]]. If not specified, it will use [[Vector.worldX]].
     *                  The y-axis will be automatically generated perpendicular to this axis and will obey the right-hand rule.
     */
    constructor(origin: Point, xAxis?: Vector);
    /**
     * Gets the point at the center of the plane.
     */
    get origin(): Point;
    /**
     * Gets the plane's x-axis.
     */
    get xAxis(): Vector;
    /**
     * Gets the plane's y-axis.
     *
     * This vector is always perpendicular to the plane's [[xAxis]] and
     * orientated to obey the right-hand rule.
     */
    get yAxis(): Vector;
    /***
     * Checks whether another plane has the same [[origin]] and [[xAxis]]. Returns true if it does.
     * @param plane       The plane to compare against.
     * @param tolerance   The amount the [[origin]] points can differ and still be considered equal.
     */
    equals(plane: Plane, tolerance?: number): boolean;
    /**
     * Remaps a point from the u-v space of the plane to the global coordinate system.
     * This is the opposite of [[remapToPlaneSpace]].
     *
     * ### Example
     * ```js
     * import { Plane, Point, Vector } from 'shapetypes'
     *
     * // Create a plane
     * const plane = new Plane(new Point(3,4), Vector.worldX());
     *
     * // Map a point from the global coordinate system to the plane's local coordinates
     * const worldPoint = new Point(5,6);
     * const planePoint = plane.remapToPlaneSpace(worldPoint);
     * console.log(planePoint.toString());
     * // => [2,2]
     *
     * // Remap the local point back to the global coordinate system
     * const output = plane.pointAt(planePoint);
     * console.log(output.toString());
     * // => [5,6]
     *
     *
     * ```
     *
     * @param u   The location of the point measured as a distance along the plane's [[xAxis]].
     * @param v   The location of the point measured as a distance along the plane's [[yAxis]].
     * @returns   The u-v point remapped to the global coordinate system.
     */
    pointAt(u: number, v: number): Point;
    /**
     * Remaps a point from the u-v space of the plane to the global coordinate system.
     * This is the opposite of [[remapToPlaneSpace]].
     * @param uvPoint   The location of the point in the plane's coordinate system.
     *                  The x coordinate of this point is the location of the point measured as a distance along the plane's [[xAxis]].
     *                  The y coordinate of this point is the location of the point measured as a distance along the plane's [[yAxis]].
     * @returns         The u-v point remapped to the global coordinate system.
     */
    pointAt(uvPoint: Point): Point;
    /**
     * Remaps a point to the u-v space of the plane.
     * This is the opposite of [[pointAt]].
     * @param point   Point to remap.
     * @returns       A point in the u-v coordinates of the plane. See [[pointAt]] for more details.
     */
    remapToPlaneSpace(point: Point): Point;
    /**
     * Creates a copy of the plane with a different [[origin]] point.
     * @param newOrigin   The origin of the new plane.
     */
    withOrigin(newOrigin: Point): Plane;
    /**
     * Creates a copy of the plane with a different [[xAxis]]. This will also
     * change the y-axis.
     * @param newXAxis    The x-axis of the new plane.
     */
    withXAxis(newXAxis: Vector): Plane;
    /***
     * Gets the plane as a string in the format: `[origin,xAxis]`.
     */
    toString(): string;
    /***
     * Transforms the plane by a [[Transform]] matrix and returns the result.
     *
     * ### Example
     * ```js
     * import { Plane, Point, Transform, Vector } from 'shapetypes';
     *
     * // Create a plane
     * const plane = new Plane(new Point(2,2), Vector.worldX());
     * console.log(plane.origin.toString());
     * // => (2,2)
     *
     * // Translate using a transform matrix
     * const mover = Transform.translate(new Vector(3,4));
     * const moved = plane.transform(mover);
     * console.log(moved.origin.toString());
     * // => (5,6)
     *
     * // Translate using the direct method
     * const otherMoved = plane.translate(new Vector(3,4));
     * console.log(otherMoved.origin.toString());
     * // => (5,6)
     * ```
     *
     * @note If you're applying the same transformation a lot of geometry,
     * creating the [[Transform]] matrix once and calling this function is faster
     * than using the direct methods.
     *
     * @category Transform
     * @param change  The [[transform]] matrix to apply to the plane.
     */
    transform(change: Transform): this;
    private getRemapTransform;
}
