import { Geometry, Transform, Vector } from '../index';
/**
 * A two-dimensional point. The point is defined by an [[x]] coordinate and a [[y]] coordinate.
 *
 * ### Example
 * ```js
 * import { Point, Vector } from 'shapetypes';
 *
 * // Create a point
 * const p = new Point(3, 4);
 *
 * // Get properties of the point
 * console.log(p.x);
 * // => 3
 *
 * // Copy the point and change a parameter
 * const other = p.withX(-1);
 * console.log(other.x);
 * // => -1
 *
 * // Measure distance between points
 * const distance = p.distanceTo(other);
 * console.log(distance);
 * // => 4
 *
 * // Translate the point
 * const shifted = p.translate(new Vector(10, 20));
 * console.log(shifted.toString());
 * // => '(13,24)'
 * ```
 */
export declare class Point extends Geometry {
    /**
     * Returns the point at [0,0].
     *
     * Equivalent to `new Point(0,0)`.
     *
     * @category Create
     */
    static origin(): Point;
    private static readonly _origin;
    private readonly _x;
    private readonly _y;
    /***
     * @param x   Value of the x coordinate.
     * @param y   Value of the y coordinate.
     */
    constructor(x: number, y: number);
    /**
     * Gets the x coordinate.
     */
    get x(): number;
    /**
     * Gets the y coordinate.
     */
    get y(): number;
    /***
     * Adds x & y coordinates of another point or vector and returns the resulting point.
     * @param addend  Point or vector to add.
     */
    add(addend: Point | Vector): Point;
    /**
     * Adds an x & y value to the point and returns the resulting point.
     * @param x       Value to add to the x coordinate.
     * @param y       Value to add to the y coordinate.
     */
    add(x: number, y: number): Point;
    /***
     * Returns the distance to another point.
     * @param point Point to measure the distance to.
     */
    distanceTo(point: Point): number;
    /***
     * Divides the x & y coordinates by a value and returns the resulting point.
     *
     * @param denominator Amount to divide the coordinates by.
     */
    divide(denominator: number): Point;
    /**
     * Divides the x & y coordinates by different amounts in the x- and y-direction. Returns the resulting point.
     * @param denominatorX    Amount to divide the x coordinate by.
     * @param denominatorY    Amount to divide the y coordinate by.
     */
    divide(denominatorX: number, denominatorY: number): Point;
    /***
     * Checks whether another point has the same [[x]] and [[y]] values. Returns true if it does.
     *
     * @param comparison  Point to compare against.
     * @param tolerance   The amount that the [[x]] and [[y]] values of the points can differ and still be considered equal.
     */
    equals(comparison: Point, tolerance?: number): boolean;
    /***
     * Multiplies the x & y coordinates by a value and returns the resulting point.
     *
     * @param factor  Amount to multiply by.
     */
    multiply(factor: number): Point;
    /**
     * Multiplies the x & y coordinates by different amounts in the x- and y-direction. Returns the resulting point.
     * @param factorX   Amount to multiply the x coordinate by.
     * @param factorY   Amount to multiply the y coordinate by.
     */
    multiply(factorX: number, factorY: number): Point;
    /***
     * Subtracts the x & y coordinates of another point or vector and returns the resulting point.
     * @param subtrahend Point or vector to subtract from the point.
     */
    subtract(subtrahend: Point | Vector): Point;
    /**
     * Subtracts an x & y value from the point and returns the resulting point.
     * @param x       Value to subtract from the x coordinate.
     * @param y       Value to subtract from the y coordinate.
     */
    subtract(x: number, y: number): Point;
    /***
     * Gets the point as a string in the format: `(x,y)`.
     */
    toString(): string;
    /**
     * Creates a copy of the vector with a different x coordinate.
     * @param newX New x coordinate.
     */
    withX(newX: number): Point;
    /**
     * Creates a copy of the vector with a different y coordinate.
     * @param newY New y coordinate.
     */
    withY(newY: number): Point;
    /***
     * Transforms the point by a [[transform]] matrix and returns the resulting point.
     *
     * ### Example
     * ```js
     * import { Point, Transform, Vector } from 'shapetypes';
     *
     * // Create a point
     * const p = new Point(3, 4);
     *
     * // Translate point using a transform matrix
     * const matrix = Transform.translate(new Vector(10, 20));
     * const transformed = p.transform(matrix);
     * console.log(transformed.toString());
     * // => (13,24)
     *
     * // Translate point using the direct method
     * const direct = p.translate(new Vector(10, 20));
     * console.log(direct.toString());
     * // => (13,24)
     * ```
     *
     * @note If you're applying the same transformation a lot of geometry,
     * creating the [[Transform]] matrix once and calling this function is faster
     * than using the direct methods.
     *
     * @category Transform
     * @param change  The [[transform]] matrix to apply to the point.
     */
    transform(change: Transform): this;
}
