import { Polygon as pcPolygon } from 'polygon-clipping';
import { BoundingBox, Geometry, Point, PointContainment, Polyline, Transform } from '../index';
/**
 * A two-dimensional shape with an outer [[boundary]] and a set of interior [[holes]] (optional).
 *
 * The boundary is always in a counter-clockwise orientation, and the holes are always clockwise.
 * If the environment's y-axis points downwards, the boundary will appear to be clockwise, and the
 * holes will appear to be counter-clockwise.
 *
 * ### Example
 * ```js
 * import { Point, Polygon, Polyline, Rectangle } from 'shapetypes'
 *
 * // Create a polygon from a triangular polyline
 * const triangle = new Polyline([new Point(0, 0), new Point(1, 1), new Point(2, 0)], true);
 * const polygon = new Polygon(triangle);
 *
 * // Get properties of the polygon
 * console.log(polygon.area);
 * // => 1
 * console.log(polygon.boundary.from.toString());
 * // => (0,0)
 *
 * // Check for point containment
 * console.log(polygon.contains(new Point(1, 0.5)));
 * // => True
 *
 * // Create a larger polygon from a rectangle
 * const outer = new Rectangle(Plane.worldXY(), 10, 10).toPolyline();
 * const outerPolygon = new Polygon(outer);
 * console.log(outerPolygon.area);
 * // => 100
 *
 * // Subtract the triangle from the rectangle
 * const subtracted = outerPolygon.difference(triangle);
 *
 * // Check that the area was reduced
 * console.log(subtracted[0].area);
 * // => 99
 *
 * // Check to see whether the point is still contained
 * console.log(subtracted[0].contains(new Point(1, 0.5)));
 * // => False
 *
 * ```
 *
 *
 */
export declare class Polygon extends Geometry {
    /**
     * Creates a polygon from a list polylines (in the GeoJSON format).
     * The first polyline is the boundary and any subsequent polylines are the holes.
     *
     * @note  May throw an error if coordinates do not form a closed polyline.
     * @category Create
     * @param coordinates   List of points in the format `[[[b_x1,b_y1],[b_x2,b_y2],...], [[h_x1,h_y1],[h_x2,h_y2],...],...]`.
     */
    static fromCoords(coordinates: pcPolygon): Polygon;
    private readonly _boundary;
    private readonly _holes;
    /***
     * Creates a polygon from an outer boundary and a list of holes.
     *
     * @note  Doesn't verify that this is a valid polygon. It is possible to
     * create a polygon with self-intersection, zero-length segments, or holes in weird places.
     * This may cause problems in some functions.
     *
     * @note  Will throw an error if the boundary or holes aren't closed.
     *
     * @param boundary  The outer edge of the polygon (must be a closed polyline).
     * @param holes     An optional list of holes cut from the interior of the polygon (must be closed polylines).
     */
    constructor(boundary: Polyline, holes?: readonly Polyline[] | undefined);
    /**
     * Gets the area of the polygon. Holes are not included in the area.
     */
    get area(): number;
    /**
     * Gets the polyline that defines the outer edge of the polygon.
     */
    get boundary(): Polyline;
    /***
     * Gets the smallest bounding box that contains the boundary of the polygon.
     *
     */
    get boundingBox(): BoundingBox;
    /**
     * Gets the list of holes (if any) that define the subtracted regions of the polygon.
     */
    get holes(): readonly Polyline[];
    /**
     * Finds the closest boundary or hole. Returns the polyline defining the closest boundary or hole.
     *
     * @param testPoint  The target to get closest to.
     */
    closestLoop(testPoint: Point): Polyline;
    /***
     * Finds the closest point on the polygon and returns the point. This point could
     * be on the edge of the [[boundary]] or one of the [[holes]],
     * or it could be a point within the interior of the polygon.
     *
     * @param testPoint  The target to get closest to.
     */
    closestPoint(testPoint: Point): Point;
    /***
     * Checks whether a point is inside, outside, or on the edge of a polygon.
     *
     * @param point       The point to test for containment.
     * @param tolerance   The distance the point can be from the edge of the polygon and still considered coincident.
     */
    contains(point: Point, tolerance?: number): PointContainment;
    /***
     * Checks whether another polygon has the same [[boundary]] and [[holes]]. Returns true if it does.
     * @param otherPolygon  Polygon to compare against.
     * @param tolerance     The amount the points can differ and still be considered equal.
     */
    equals(otherPolygon: Polygon, tolerance?: number): boolean;
    /***
     * Gets the polygon as a string in the format: `[boundary, ...holes]`.
     */
    toString(): string;
    /**
     * Joins this polygon with another polygon or closed polyline. Returns the result.
     * @param joiner  Either a closed polyline, polygon, or a list of the two.
     * @returns      The polygon created when the two objects were joined.
     *                In some cases, this may return multiple polygons if the objects don't overlap.
     */
    union(joiner: Polyline | Polygon | readonly Polyline[] | readonly Polygon[]): ReadonlyArray<Polygon>;
    /**
     * Intersects this polygon with another polygon or closed polyline. Returns the overlapping portion.
     * @param intersector   Either a closed polyline, polygon, or a list of the two.
     * @returns       The overlapping portion of the two objects.
     *                In some cases, this may return an empty list if the polygons don't overlap.
     */
    intersection(intersector: Polyline | Polygon | readonly Polyline[] | readonly Polygon[]): ReadonlyArray<Polygon>;
    /**
     * Subtracts a polygon or closed polyline from this polygon. Returns the part left over.
     * @param subtractor    Either a closed polyline, polygon, or a list of the two.
     * @returns       The part left over.
     */
    difference(subtractor: Polyline | Polygon | readonly Polyline[] | readonly Polygon[]): ReadonlyArray<Polygon>;
    /**
     * Gets the polygon in the GeoJSON format: `[[[b_x1,b_y1],[b_x2,b_y2],...], [[h_x1,h_y1],[h_x2,h_y2],...],...]`.
     */
    asGeoJSON(): pcPolygon;
    /***
     * Transforms the polyline by a [[transform]] matrix and returns the result.
     *
     * ### Example
     * ```js
     *
     * import { Point, Polygon, Polyline, Transform, Vector } from 'shapetypes';
     *
     * // Create a new polygon from a triangular polyline
     * const triangle = new Polyline([new Point(0, 0), new Point(1, 1), new Point(2, 0)], true);
     * const polygon = new Polygon(triangle);
     * console.log(polygon.boundary.from.toString());
     * // => (0,0)
     *
     * // Translate polygon using a transform matrix
     * const matrix = Transform.translate(new Vector(3,4));
     * const shifted = polygon.transform(matrix);
     * console.log(shifted.boundary.from.toString());
     * // => (3,4)
     *
     * // Translate polygon using the direct method
     * const otherShifted = polygon.translate(new Vector(3, 4));
     * console.log(otherShifted.boundary.from.toString());
     * // => (3,4)
     * ```
     *
     * @note If you're applying the same transformation a lot of geometry,
     * creating the [[Transform]] matrix once and calling this function is faster
     * than using the direct methods.
     *
     * @category Transform
     * @param change  The [[transform]] matrix to apply to the polygon.
     */
    transform(change: Transform): this;
}
