import { Ring } from 'polygon-clipping';
import { CurveOrientation, PointContainment } from './utilities';
import { BoundingBox, Circle, Geometry, Interval, IntervalSorted, Line, Point, Polygon, Ray, Rectangle, Transform, Vector } from '../index';
/**
 * A continuous line made from a series of straight [[segments]].
 * An array of [[points]] defines the corners of the polyline.
 *
 * If the polyline's start and end point are the same, it is considered closed.
 * This can be checked with [[isClosed]].
 * Some operations, such as [[contains]], only work with closed polylines.
 *
 * ### Example
 * ```js
 * import { Point, Polyline, Vector } from 'shapetypes'
 *
 * // Create a new polyline
 * const triangle = new Polyline([new Point(0, 0), new Point(1, 1), new Point(2, 0)], true);
 *
 * // Get properties of the polyline
 * console.log(triangle.area);
 * // => 1
 * console.log(triangle.length);
 * // => 4.82842712474619
 *
 * // Check whether a point is inside the closed polyline
 * console.log(triangle.contains(new Point(1, 0.5)));
 * // => True
 *
 * // Move the polyline and check whether the point is inside
 * const shifted = triangle.translate(new Vector(3, 4));
 * console.log(shifted.contains(new Point(1, 0.5)));
 * // => False
 * ```
 */
export declare class Polyline extends Geometry {
    /**
     * Creates a polyline from a list of corner coordinates (in the GeoJSON format).
     * @category Create
     * @param coordinates   List of points in the format `[[x1,y1],[x2,y2],[x3,y3],...]`
     * @param makeClosed    If true, checks whether the resulting polyline [[isClosed]] and if it isn't, adds another point ensuring the polyline ends the same place it starts.
     */
    static fromCoords(coordinates: Ring, makeClosed?: boolean): Polyline;
    private readonly _points;
    private _cacheBoundingBox;
    private _cacheGeoJSON;
    private _cacheArea;
    private _cacheClosed;
    private _cacheLength;
    private _cacheOrientationYUp;
    private _cacheOrientationYDown;
    private _cacheSegments;
    /***
     * Creates a polyline from a set of corner points.
     *
     * @note  Doesn't verify that this is a valid polyline. It is possible to
     * create a polyline with self-intersection or zero-length segments, which may cause problems in some functions.
     *
     * @param points        The points defining the corners of the polyline.
     * @param makeClosed    If true, checks whether the resulting polyline [[isClosed]] and if it isn't, adds another point ensuring the polyline ends the same place it starts.
     */
    constructor(points: readonly Point[], makeClosed?: boolean);
    /**
     * Gets the area enclosed by the polyline. If the polyline isn't closed, returns 0.
     */
    get area(): number;
    /***
     * Gets the smallest bounding box that contains the polyline.
     */
    get boundingBox(): BoundingBox;
    /**
     * Gets the number of points in the polyline.
     */
    get count(): number;
    /**
     * Gets the starting point of the polyline.
     */
    get from(): Point;
    /**
     * Checks whether the polyline starts and ends in the same place (that [[from]] equals [[to]]).
     * Returns true if it does.
     */
    get isClosed(): boolean;
    /**
     * Gets the length of the polyline.
     */
    get length(): number;
    /**
     * Gets the points that define the corners of the polyline.
     */
    get points(): readonly Point[];
    /**
     * Gets the number of segments in the polyline.
     */
    get segmentCount(): number;
    /**
     * Gets the segments (or edges) of the polyline.
     */
    get segments(): readonly Line[];
    /**
     * Gets the end of the polyline.
     */
    get to(): Point;
    /**
     * Calculates the weighted average of all segments and returns the result.
     */
    center(): Point;
    /**
     * Finds the closest point in the list of [[points]] and returns its index.
     * @param testPoint   The target to get closest to.
     */
    closestIndex(testPoint: Point): number;
    /***
     * Finds the closest point on the polyline and returns the parameter for the point.
     *
     * @param testPoint   The target to get closest to.
     * @returns           The parameter of the closest point. Entering the parameter into [[pointAt]] will return the closest point.
     */
    closestParameter(testPoint: Point): number;
    /***
     * Finds the closest point on the polyline and returns the point.
     * @param testPoint       Target to get closest to.
     * @param includeInterior If false, the closest point must lie on a segment of the polyline.
     *                        If true, the closest point can also be a point on the polyline's interior (if it is closed and has an interior).
     */
    closestPoint(testPoint: Point, includeInterior?: boolean): Point;
    /**
     * Creates a copy of the polyline with short segments removed. This is achieved by
     * traveling along the polyline and removing any point closer to the previous
     * point than `minimumSegmentSize`. Does not remove [[from]] or [[to]] points.
     * @param minimumSegmentSize   Shortest allowable segment.
     */
    deleteShortSegments(minimumSegmentSize?: number): Polyline;
    /***
     * Checks whether another polyline has the same points. Returns true if it does.
     * @param otherPolyline   Polyline to compare against.
     * @param tolerance       The amount the points can differ and still be considered equal.
     */
    equals(otherPolyline: Polyline, tolerance?: number): boolean;
    /***
     * Calculates where the polyline intersects other geometry and returns the parameters
     * for these points of intersection.
     *
     * @note              This is an alias for the [[Intersection.polyline]] function.
     * @note              Only accounts for crossings, not coincident overlaps.
     *
     * @param otherGeom   The geometry to intersect.
     * @returns           The parameters of the intersection points.
     *                    The array is always sorted from the smallest to largest parameter.
     *                    Entering the parameter into [[pointAt]] will return the intersection point.
     */
    intersection(otherGeom: Point | Line | Ray | BoundingBox | Circle | Rectangle | Polyline | Polygon | ReadonlyArray<Point | Line | Ray | BoundingBox | Circle | Rectangle | Polyline | Polygon>): readonly number[];
    /**
     * Returns a closed copy of the polyline.
     * If the polyline is already closed, returns itself.
     * If the polyline is open, closes it by adding a segment between [[to]] and [[from]].
     */
    makeClosed(): Polyline;
    /**
     * Creates a copy of the polyline with colinear segments merged together.
     * This is achieved by traveling along the polyline and merging any segments that travel in the same direction.
     * @param angleTolerance  The minimum angle at which segments are no longer colinear (in radians).
     * @param includeSeam     If true, it will also test seam on closed polylines for colinearity. If they are colinear, will move [[from]] and [[to]] points.
     */
    mergeColinearSegments(angleTolerance?: number, includeSeam?: boolean): Polyline;
    /**
     * Calculates the normal to the polyline at a given parameter.
     *
     * If the polyline is closed, it will always return the normal that points towards the inside.
     * If the polyline is open, the vector will be on the left side of the segment (which is the right side if the environment's y-axis points downwards).
     *
     * @note  At corners where two segments join, returns the normal to the segment with the smallest index.
     *
     * @param u   Position on the polyline to calculate the normal (see [[pointAt]] for explanation)
     * @returns   A unit vector
     */
    normalAt(u: number): Vector;
    /**
     * Calculates whether a polyline is in a clockwise or counter-clockwise orientation and returns the result.
     * If the polyline is not closed, it can't have an orientation, so the function returns [[CurveOrientation.undefined]].
     *
     * @param isYDown   The function assumes that the environment's y-axis points upwards.
     *                  In environments where the y-axis points downwards, `isYDown` should be set to `true`.
     *                  This inverts clockwise and counter-clockwise to ensure that
     *                  they are in the correct orientation for how you're viewing them.
     */
    orientation(isYDown?: boolean): CurveOrientation;
    /**
     * Finds the point a normalized parameter along the polyline. Returns the point.
     *
     * Each segment of the polyline is parameterized from 0 to 1 (like [[Line.pointAt]]).
     * So `0` is the start of the first segment, and `1` is the end.
     * `0.5` is the mid point of the first segment, and `1.5` is the mid point of the second segment.
     *
     * ### Example
     * ```js
     * const triangle = new Polyline([new Point(0, 0), new Point(1, 1), new Point(2, 0)], true);
     *
     * // The start of first segment
     * console.log(triangle.pointAt(0).toString());
     * // => (0,0)
     *
     * // Mid point of first segment
     * console.log(triangle.pointAt(0.5).toString());
     * // => (0.5,0.5)
     *
     * // The end of first segment / start of second segment
     * console.log(triangle.pointAt(1).toString());
     * // => (1,1)
     *
     * // Mid point of second segment
     * console.log(triangle.pointAt(1.5).toString());
     * // => (1.5,0.5)
     *
     * ```
     *
     * @param u   The normalized parameter.
     */
    pointAt(u: number): Point;
    /**
     * Gets a segment of the polyline.
     *
     * @param index   The index of the segment.
     */
    segmentAt(index: number): Line;
    /**
     * Extracts a section of the polyline. Returns the resulting polyline.
     * @param domain  The section of the polyline to extract. The domain contains
     * two numbers, the `min`, which is the parameter for the start of the new polyline.
     * And the `max`, which will be the end. Everything between these two points is included
     * in the new polyline. The position of the domain is calculated using the same method as [[pointAt]].
     */
    trim(domain: IntervalSorted | Interval): Polyline;
    /**
     * Creates a copy of the polyline with the points in reverse order.
     */
    reverse(): Polyline;
    /**
     * Gets the polyline as a string in the format: `[(x1,y1),(x2,y2),(x3,y3)]`.
     */
    toString(): string;
    /***
     * Checks whether a point is inside, outside, or on the edge of a polyline.
     *
     * @note This method only works with closed polylines. It will throw an error if applied to an open polyline.
     *
     * @category Closed
     * @param point       The point to test for containment.
     * @param tolerance   The distance the point can be from the edge of the polyline and still considered coincident.
     */
    contains(point: Point, tolerance?: number): PointContainment;
    /**
     * Returns a copy of the polyline with a given [[CurveOrientation]].
     * If the polyline already has that orientation, returns self.
     *
     * @note This method only works with closed polylines. It may throw an error if applied to an open polyline.
     *
     * @category Closed
     * @param goal      The desired orientation of the new polyline.
     */
    withOrientation(goal: CurveOrientation, isYDown?: boolean): Polyline;
    /**
     * Gets the polyline in the GeoJSON format: `[[x1,y1],[x2,y2],...]`.
     */
    asGeoJSON(): Ring;
    /***
     * Transforms the polyline by a [[transform]] matrix and returns the result.
     *
     * ### Example
     * ```js
     * import { Point, Polyline. Transform, Vector } from 'shapetypes';
     *
     * // Create a new polyline
     * const triangle = new Polyline([new Point(0, 0), new Point(1, 1), new Point(2, 0)], true);
     *
     * // Check start position of the polyline
     * console.log(shifted.from.toString());
     * // => (0,0)
     *
     * // Translate polyline using a transform matrix
     * const matrix = Transform.translate(new Vector(3,4));
     * const shifted = triangle.transform(matrix);
     * console.log(shifted.from.toString());
     * // => (3,4)
     *
     * // Translate polyline using the direct method
     * const otherShifted = triangle.translate(new Vector(3, 4));
     * console.log(otherShifted.from.toString());
     * // => (3,4)
     * ```
     *
     * @note If you're applying the same transformation a lot of geometry,
     * creating the [[Transform]] matrix once and calling this function is faster
     * than using the direct methods.
     *
     *
     * @category Transform
     * @param change  The [[transform]] matrix to apply to the polyline.
     */
    transform(change: Transform): this;
}
