import { BoundingBox, Geometry, Interval, IntervalSorted, Line, Plane, Point, PointContainment, Polyline, Transform } from '../index';
/**
 * A rectangular shape aligned to a plane.
 *
 * ### Example
 * ```js
 * import { Plane, Point, Rectangle, Vector } from 'shapetypes';
 *
 * // Create a new rectangle
 * const rect = new Rectangle(Plane.worldXY(), 10, 20);
 *
 * // Get properties of the rectangle
 * console.log(rect.area);
 * // => 200
 * console.log(rect.circumference);
 * // => 60
 * console.log(rect.center.toString());
 * // => (5,10)
 *
 * // Create a new rectangle aligned to a plane
 * const plane = new Plane(new Point(3,4), new Vector(1,1));
 * const angledRect = new Rectangle(plane, 10, 20);
 *
 * // Get corners of angled rectangle
 * console.log(angledRect.getCorners());
 * // => [(3,4),(-11.14213562373095,8.14213562373095),(-4.071067811865474,25.213203435596427),(10.071067811865476,11.071067811865476)]
 * ```
 */
export declare class Rectangle extends Geometry {
    /**
     * Creates a rectangle from two corner points.
     * @category Create
     * @param cornerA   One corner of the rectangle.
     * @param cornerB   The opposite corner of the rectangle.
     * @param plane     The plane the rectangle is orientated to.
     *                  If unset, the rectangle will be aligned to the global x- and y-axis.
     */
    static fromCorners(cornerA: Point, cornerB: Point, plane?: Plane): Rectangle;
    /**
     * Creates a rectangle centered and aligned to a plane.
     *
     * @category Create
     * @param x       Width of the rectangle along the x-axis of the plane (rectangle will be centered on the plane's origin).
     * @param y       Width of the rectangle along the y-axis of the plane (rectangle will be centered on the plane's origin).
     * @param center  The plane the rectangle is aligned to. The plane's origin will be the center of the rectangle.
     */
    static fromCenter(center: Plane, x: number, y: number): Rectangle;
    private readonly _plane;
    private readonly _x;
    private readonly _y;
    /***
     * Creates a rectangle aligned to a plane. One corner of the rectangle will
     * sit on the plane's origin, with the rest of the rectangle growing along the
     * plane's axes from that point.
     *
     *
     * @param plane   The plane the rectangle is aligned to. The plane's origin will be one corner of the rectangle
     *                with the rest of the rectangle growing from that point.
     * @param x   The width of the rectangle along the plane's x-axis.
     * @param y   The width of the rectangle along the plane's y-axis.
     */
    constructor(plane: Plane, x: number, y: number);
    /**
     * Creates a new rectangle on a plane.
     * @param plane   The position of the rectangle and its axes.
     * @param x       Position of the rectangle along the x-axis of the plane.
     * @param y       Position of the rectangle along the y-axis of the plane.
     */
    constructor(plane: Plane, x: Interval | IntervalSorted, y: Interval | IntervalSorted);
    /**
     * Gets the area of the rectangle.
     */
    get area(): number;
    /***
     * Gets the smallest bounding box that contains the rectangle.
     */
    get boundingBox(): BoundingBox;
    /**
     * Gets the center of the rectangle.
     */
    get center(): Point;
    /**
     * Gets the total length of the rectangle's edges.
     */
    get circumference(): number;
    /**
     * Gets the plane the rectangle is aligned to.
     *
     * Depending on how the rectangle is positioned on this plane
     * (using the intervals [[x]] and [[y]]), the plane's origin may or may not be
     * a corner or center of the rectangle.
     */
    get plane(): Plane;
    /**
     * Gets the width of the rectangle along its x-axis.
     */
    get widthX(): number;
    /**
     * Gets the width of the rectangle along its y-axis.
     */
    get widthY(): number;
    /**
     * Gets the position of the rectangle relative to the [[plane]]'s x-axis.
     */
    get x(): IntervalSorted;
    /**
     * Gets the position of the rectangle relative to the [[plane]]'s y-axis.
     */
    get y(): IntervalSorted;
    /***
     * Finds the closest point on the rectangle and returns the point.
     * @param testPoint       The target to get closest to.
     * @param includeInterior If false, the closest point must lie on the outer edge of the rectangle.
     *                        If true, the closest point can also be a point on the interior of the rectangle.
     */
    closestPoint(testPoint: Point, includeInterior?: boolean): Point;
    /***
     * Checks whether a point is inside, outside, or on the edge of a rectangle.
     *
     * @param testPoint   The point to test for containment.
     * @param tolerance   The distance the point can be from the edge of the rectangle and still considered coincident.
     */
    contains(testPoint: Point, tolerance?: number): PointContainment;
    /**
     * Gets a corner of the rectangle.
     *
     * @param minX  If true, point will be at the min x value relative to the rectangle's plane. If false, it will be at max.
     * @param minY  If true, point will be at the min y value. If false, it will be at max.
     */
    corner(minX: boolean, minY: boolean): Point;
    /***
     * Checks whether another rectangle has the same dimensions and location. Returns true if it does.
     * @param otherRectangle    Rectangle to compare against.
     * @param tolerance         The amount the dimensions can differ and still be considered equal.
     */
    equals(otherRectangle: Rectangle, tolerance?: number): boolean;
    /***
     * Gets the four corners of the rectangle.
     * The order is: [[minX, minY], [minX, maxY], [maxX, maxY], [maxX, minY]].
     */
    getCorners(): readonly Point[];
    /**
     * Gets the four edges of the rectangle. Follows the order in [[getCorners]].
     */
    getEdges(): readonly Line[];
    /***
     * Remaps a point from the u-v space of the rectangle to the global coordinate system.
     * @param u         The normalized distance along the x-axis of the rectangle.
     * @param v         The normalized distance along the y-axis of the rectangle.
     * @returns         The uvPoint remapped to the global coordinate system.
     */
    pointAt(u: number, v: number): Point;
    /**
     * Remaps a point from the u-v space of the rectangle to the global coordinate system.
     * @param uvPoint   A point in the u-v coordinates of the rectangle.
     *                  The point's x value is the normalized distance along the x-axis of the rectangle (u-direction).
     *                  The point's y value is the normalized distance along the y-axis of the rectangle (v-direction).
     * @returns         The uvPoint remapped to the global coordinate system.
     */
    pointAt(uvPoint: Point): Point;
    /***
     * Gets the rectangle as a string in the format: `[plane,widthX,widthY]`.
     */
    toString(): string;
    /***
     * Gets the rectangle as a closed polyline.
     */
    toPolyline(): Polyline;
    /**
     * Creates a copy of the rectangle with a different [[plane]].
     *
     * @param newPlane    The new plane for the rectangle. This will change the rectangle's location and orientation.
     */
    withPlane(newPlane: Plane): Rectangle;
    /**
     * Creates a copy of the rectangle with a different [[x]] interval.
     * @param newX    The new position of the rectangle along the x-axis of the plane.
     */
    withX(newX: IntervalSorted): Rectangle;
    /**
     * Creates a copy of the rectangle with a different [[y]] interval.
     * @param newY    The new position of the rectangle along the y-axis of the plane.
     */
    withY(newY: IntervalSorted): Rectangle;
    /***
     * Transforms the rectangle by a [[transform]] matrix and returns the result.
     *
     * ### Example
     * ```js
     * import { Plane, Rectangle, Transform } from 'shapetypes';
     *
     * // Create a new rectangle
     * const rect = new Rectangle(Plane.worldXY(), 10, 20);
     * console.log(rect.area);
     * // => 200
     *
     * // Scale rectangle using a transform matrix
     * const matrix = Transform.scale(2);
     * const scaled = rect.transform(matrix);
     * console.log(scaled.area);
     * // => 800
     *
     * // Scale rectangle using the direct method
     * const otherScaled = rect.scale(2);
     * console.log(otherScaled.area);
     * // => 800
     * ```
     *
     * @note If you're applying the same transformation a lot of geometry,
     * creating the [[Transform]] matrix once and calling this function is faster
     * than using the direct methods.
     *
     * @category Transform
     * @param change  The [[transform]] matrix to apply to the rectangle.
     */
    transform(change: Transform): this;
}
