import { Plane, Point, Vector } from '../index';
/**
 * A 3x3 transformation matrix that is used to rotate, scale, and translate geometry.
 *
 *
 * In many cases, you don't need to create this transformation matrix yourself,
 * you can instead use the transformation functions available to all [[Geometry]] objects.
 * That said, in some cases, you may still want to create your own matrix, particularly if you're
 * doing your own custom transformations, or you're applying the same transformation
 * many times. All geometry objects have a method to apply a transformation
 * matrix (see: [[Vector.transform]] as an example).
 *
 *
 * ### Example
 * ```js
 * import { Transform, Vector } from 'shapetypes';
 *
 * // Create a new vecctor
 * const vector = new Vector(3, 4);
 * console.log(vector.length);
 * // => 5
 *
 * // Scale vector using a transform matrix
 * const matrix = Transform.scale(2);
 * const scaled = vector.transform(matrix);
 * console.log(vector.length);
 * // => 10
 *
 * // Scale vector using the direct method
 * const otherScaled = vector.scale(2);
 * console.log(otherScaled.length);
 * // => 10
 * ```
 *
 */
export declare class Transform {
    /**
     * Creates a transform matrix where the diagonal elements ([[M00]], [[M11]], [[M22]]) are
     * set to one value and the other elements are set to another value.
     *
     * @category Create
     * @param base        The value for all elements except [[M00]], [[M11]], [[M22]].
     * @param diagonal    The value for elements [[M00]], [[M11]], [[M22]].
     */
    static fromDiagonal(base?: number, diagonal?: number): Transform;
    /**
     * Creates a transform matrix where the diagonal elements ([[M00]], [[M11]], [[M22]]) are
     * set to 1 and the other elements are set to 0.
     * @category Create
     */
    static identity(): Transform;
    /***
     * Creates a transform matrix that transforms the geometry from one coordinate system
     * to another while keeping the geometry in the same position.
     * In other words, if the geometry is described relative to `planeFrom`, after
     * applying this translation, it will be in the same position but described relative to `planeTo`.
     *
     * ### Example
     * ```js
     * import { Plane, Point, Transform, Vector } from 'shapetypes';
     *
     * // Setup the transformation
     * const from = new Plane(new Point(3, 4), Vector.worldX());
     * const to = Plane.worldXY();
     * const tran = Transform.changeBasis(from, to);
     *
     * // Change the point's coordinates from the `from` plane to the `to` plane.
     * const p = new Point(1, 2);
     * const after = tran.transformPoint(p);
     * console.log(after.toString());
     * // => [4,6]
     * ```
     *
     * @category Create
     * @param planeFrom   The coordinate system the geometry is currently described in.
     * @param planeTo     The coordinate system to describe the geometry in.
     */
    static changeBasis(planeFrom: Plane, planeTo: Plane): Transform;
    /***
     * Creates a transform matrix that moves the geometry from one plane to another.
     * The resulting geometry will be in the same place relative to `planeTo` as it was relative to `planeFrom`.
     *
     * ### Example
     * ```js
     * import { Plane, Point, Transform, Vector } from 'shapetypes';
     *
     * // Setup the transformation
     * const from = new Plane(new Point(3, 4), Vector.worldX());
     * const to = Plane.worldXY();
     * const tran = Transform.planeToPlane(from, to);
     *
     * // Move point so it is in the same place relative to plane `to` as it was for `from`
     * const p = new Point(4, 6);
     * const after = tran.transformPoint(p);
     * console.log(after.toString());
     * // => (1,2)
     * ```
     *
     * @category Create
     * @param planeFrom   The plane to move from.
     * @param planeTo     The plane to move to.
     */
    static planeToPlane(planeFrom: Plane, planeTo: Plane): Transform;
    /***
     * Creates a transform matrix that rotates an object about a point.
     *
     * @category Create
     * @param angle   The angle of rotation (in radians).
     *                If the environment's y-axis points upwards, the direction is counter-clockwise.
     * @param pivot   The pivot point for rotation. If undefined, the object will be rotated about 0,0.
     */
    static rotate(angle: number, pivot?: Point | undefined): Transform;
    /**
     * Creates a transform matrix that scales the geometry by a specified amount along the x- and y-axis.
     *
     * @category Create
     * @param x   The amount to scale the object along the x-axis. If less than 1, the object will shrink. If greater than 1, it will grow.
     * @param y   The amount to scale the object along the y-axis. If undefined, uses value from `x`.
     * @param center    The center of scaling. All objects will shrink towards and grow away from this point. If undefined, it will use 0,0.
     */
    static scale(x: number, y?: number, center?: Point): Transform;
    /***
     * Creates a transform matrix that moves an object along a vector.
     *
     * @category Create
     * @param move        The direction to move the object.
     * @param distance    The distance to move the object. If set to undefined, it will use length of `move` vector.
     */
    static translate(move: Vector, distance?: number | undefined): Transform;
    private static readonly _identity;
    /**
     * Returns a new Transform matrix that translates an object from the worldXY coordinate system to another system.
     * This is the same as Transfrom.changeBasis(Plane.worldXY(), planeTo);
     * @param planeTo   The plane to describe the object relative to
     */
    private static worldToPlane;
    private readonly _matrix;
    /***
     * Creates a 3x3 transformation matrix.
     *
     * The constructor specifies the 9 entries of the matrix. Conceptually, the matrix looks like:
     * - `M00 M10 M20`
     * - `M01 M11 M21`
     * - `M02 M12 M22`
     *
     * @param M00
     * @param M10
     * @param M20
     * @param M01
     * @param M11
     * @param M21
     * @param M02
     * @param M12
     * @param M22
     */
    constructor(M00: number, M10: number, M20: number, M01: number, M11: number, M21: number, M02: number, M12: number, M22: number);
    /**
     * Gets the determinant of the matrix.
     */
    get determinant(): number;
    get M00(): number;
    get M10(): number;
    get M20(): number;
    get M01(): number;
    get M11(): number;
    get M21(): number;
    get M02(): number;
    get M12(): number;
    get M22(): number;
    /***
     * Checks whether another transform matrix has exactly the same values. Returns true if it does.
     * @param otherMatrix The matrix to compare against.
     */
    equals(otherMatrix: Transform): boolean;
    /**
     * Inverts the matrix and returns the resulting matrix. Generally, applying the inverse
     * matrix to an object will undo the impact of the original matrix. So if the
     * original matrix rotated the object 90 degrees, the inverted matrix will rotate
     * the object 90 degrees in the opposite direction.
     *
     * @note  In some cases, it isn't possible to invert the matrix.
     * If it isn't possible, the `success` return value will be `false`.
     */
    inverse(): {
        /** If the inversion was successful, it will be `true`. */
        readonly success: boolean;
        /** If the inversion was successful, it will contain the inverted matrix. Otherwise contains the original matrix. */
        readonly result: Transform;
    };
    /**
     * Multiplies this matrix by another matrix and returns the resulting matrix.
     * @param factor    The matrix to multiply by.
     */
    multiply(factor: Transform): Transform;
    /***
     * Gets the matrix as a string in the format: `[M00,M10,M20,M01,M11,M21,M02,M12,M22]`.
     */
    toString(): string;
    /**
     * Creates a copy of the Transform matrix with certain values replaced.
     * @param M00   The new value for M00. If undefined, it will use the existing value in the matrix.
     * @param M10   The new value for M10. If undefined, it will use the existing value in the matrix.
     * @param M20   The new value for M20. If undefined, it will use the existing value in the matrix.
     * @param M01   The new value for M01. If undefined, it will use the existing value in the matrix.
     * @param M11   The new value for M11. If undefined, it will use the existing value in the matrix.
     * @param M21   The new value for M21. If undefined, it will use the existing value in the matrix.
     * @param M02   The new value for M02. If undefined, it will use the existing value in the matrix.
     * @param M12   The new value for M12. If undefined, it will use the existing value in the matrix.
     * @param M22   The new value for M22. If undefined, it will use the existing value in the matrix.
     */
    withValues(M00: number | undefined, M10: number | undefined, M20: number | undefined, M01: number | undefined, M11: number | undefined, M21: number | undefined, M02: number | undefined, M12: number | undefined, M22: number | undefined): Transform;
    /**
     * @hidden
     * Returns a copy of the point transformed by this Transform matrix
     * @param point
     */
    transformPoint(point: Point): Point;
    /**
     * @hidden
     * Returns a copy of the list of points with each being transformed by this Transform matrix
     * @param points
     */
    transformPoints(points: readonly Point[]): readonly Point[];
    /**
     * @hidden
     * Returns a copy of the vector transformed by this Transform matrix. Note that vectors aren't impacted by translation transformations.
     * @param vector
     */
    transformVector(vector: Vector): Vector;
}
