import { Geometry, Interval, IntervalSorted, Line, Point, Polyline, Transform } from '../index';
/**
 * A rectangle aligned to the environment's x- and y-axis ([[Vector.xAxis]].
 * A bounding box is defined by two [[IntervalSorted]]s, which give the dimensions of the rectangle along the x- and y-axis.
 *
 * ### Example
 * ```js
 * import { BoundingBox, IntervalSorted, Point } from 'shapetypes';
 *
 * // Create a new bounding box
 * const bb = new BoundingBox(new IntervalSorted(0, 10), new IntervalSorted(5, 25));
 *
 * // Get properties of the bounding box
 * console.log(bb.area);
 * // => 200
 * console.log(bb.center.toString());
 * // => '(5,15)'
 *
 * // Test for containment
 * console.log(bb.contains(new Point(5, 15)));
 * // => True
 *
 * // Change the bounding box's xRange
 * const shifted = bb.withXRange(new IntervalSorted(0, 20));
 * console.log(shifted.area);
 * // => 400
 *
 * // Scale the bounding box
 * const scaled = bb.scale(2);
 * console.log(scaled.area);
 * // => 800
 * ```
 */
export declare class BoundingBox extends Geometry {
    /**
     * Creates a bounding box from two corner points.
     * @category Create
     * @param cornerA One corner of the bounding box.
     * @param cornerB The opposite corner of the bounding box.
     */
    static fromCorners(cornerA: Point, cornerB: Point): BoundingBox;
    /**
     * Returns the smallest bounding box that contains all the points.
     * @category Create
     * @param points  The points to encapsulate in a bounding box.
     */
    static fromPoints(points: readonly Point[]): BoundingBox;
    /**
     * Returns the smallest bounding box that contains two other bounding boxes.
     * @category Create
     * @param a   The first bounding box to encapsulate.
     * @param b   The second bounding box to encapsulate.
     */
    static union(a: BoundingBox, b: BoundingBox): BoundingBox;
    /**
     * Finds the overlapping portion of two bounding boxes and returns it as a new bounding box.
     * @category Create
     * @param a   The first bounding box to intersect
     * @param b   The second bounding box to intersect
     * @returns: A bounding box representing the overlap between these two boxes. If no overlap exists, returns undefined.
     */
    static intersection(a: BoundingBox, b: BoundingBox): BoundingBox | undefined;
    private readonly _xRange;
    private readonly _yRange;
    /**
     * Create a bounding box from two intervals.
     * @param xRange  The dimensions of the bounding box along the environment's x-axis ([[Vector.xAxis]]). The box will be from xRange.min to xRange.max.
     * @param yRange  The dimensions of the bounding box along the environment's y-axis.
     */
    constructor(xRange: IntervalSorted | Interval, yRange: IntervalSorted | Interval);
    /**
     * Gets the area of the bounding box.
     */
    get area(): number;
    /**
     * Gets the point at the center of the bounding box.
     */
    get center(): Point;
    /**
     * Gets the corner of the bounding box that has the smallest x and y values.
     */
    get min(): Point;
    /**
     * Gets the corner of the bounding box that has the largest x and y values.
     */
    get max(): Point;
    /**
     * Gets the position of the bounding box along the environment's x-axis ([[Vector.xAxis]]).
     */
    get xRange(): IntervalSorted;
    /**
     * Gets the position of the bounding box along the environment's x-axis ([[Vector.xAxis]]).
     */
    get yRange(): IntervalSorted;
    /***
     * Finds the closest point on the bounding box and returns the point.
     * @param testPoint       Target to get closest to.
     * @param includeInterior If true, the closest point can be within the bounding box. If false, the closest point can only be on the bounding box's outer edge.
     */
    closestPoint(testPoint: Point, includeInterior?: boolean): Point;
    /**
     * Checks whether a point is inside the bounding box.
     * @param testPoint   Point to test for containment
     * @param strict      If true, the point needs to be entirely inside the bounding box
     *                    and can't be coincident with the edge.
     * @param tolerance   The distance the point can be outside the box and still considered inside.
     */
    contains(testPoint: Point, strict?: boolean, tolerance?: number): boolean;
    /**
     * Gets one of the bounding box's four corners.
     * @param minX  If true, point will be at the min x value. If false, it will be at max.
     * @param minY  If true, point will be at the min y value. If false, it will be at max.
     */
    corner(minX: boolean, minY: boolean): Point;
    /***
     * Gets the four corners of the bounding box.
     * The order is: [[minX, minY], [minX, maxY], [maxX, maxY], [maxX, minY]].
     */
    getCorners(): readonly Point[];
    /**
     * Gets the four edges of the bounding box. Follows the order in [[getCorners]].
     */
    getEdges(): readonly Line[];
    /**
     * Evenly increases the size of the bounding box in all directions. Returns the enlarged bounding box.
     * @param amount  The amount to inflate each side of the bounding box.
     */
    inflate(amount: number): BoundingBox;
    inflate(amountX: number, amountY: number): BoundingBox;
    /**
     * Checks whether the bounding box overlaps another. Returns true if it does.
     * @param otherBox    The box to check for overlap.
     */
    overlaps(otherBox: BoundingBox): boolean;
    /***
     * Remaps a point from the u-v space of the bounding box to the global coordinate system.
     * This is the opposite of [[remapToBox]].
     *
     * @param u         The normalized distance along the x-axis of the bounding box.
     * @param v         The normalized distance along the y-axis of the bounding box.
     * @returns         The uvPoint remapped to the global coordinate system.
     */
    pointAt(u: number, v: number): Point;
    /**
     * Remaps a point from the u-v space of the bounding box to the global coordinate system.
     * This is the opposite of [[remapToBox]].
     * @param uvPoint   A point in the u-v coordinates of the bounding box.
     *                  The point's x value is the normalized distance along the x-axis of the bounding box (u-direction).
     *                  The point's y value is the normalized distance along the y-axis of the bounding box (v-direction).
     * @returns         The uvPoint remapped to the global coordinate system.
     */
    pointAt(uvPoint: Point): Point;
    /***
     * Remaps a point to the u-v space of the bounding box.
     * This is the opposite of [[pointAt]].
     * @param point   Point to remap.
     * @returns       A point in the u-v coordinates of the bounding box. See [[pointAt]] for more details.
     */
    remapToBox(point: Point): Point;
    /***
     * Checks whether another bounding box has the same [[xRange]] and [[yRange]]. Returns true if it does.
     * @param otherBoundingBox  The bounding box to compare against.
     */
    equals(otherBoundingBox: BoundingBox): boolean;
    /***
     * Gets the edge of the bounding box as a closed polyline.
     */
    toPolyline(): Polyline;
    /***
     * Gets the bounding box as a string in the format: `[xRange,yRange]`.
     */
    toString(): string;
    /**
     * Creates a copy of the bounding box with a different [[xRange]].
     * @param newXRange The xRange of the new bounding box.
     */
    withXRange(newXRange: IntervalSorted): BoundingBox;
    /**
     * Creates a copy of the bounding box with a different [[yRange]].
     * @param newYRange The yRange of the new bounding box.
     */
    withYRange(newYRange: IntervalSorted): BoundingBox;
    /***
     * Transforms the bounding box by a [[transform]] matrix and returns the result.
     *
     * ### Example
     * ```js
     * import { BoundingBox, IntervalSorted, Transform } from 'shapetypes';
     *
     * // Create bounding box
     * const bb = new BoundingBox(new IntervalSorted(0, 10), new IntervalSorted(5, 25));
     * console.log(bb.area);
     * // => 200
     *
     * // Scale using a transform matrix
     * const matrix = Transform.scale(2);
     * const scaled = bb.transform(matrix);
     * console.log(scaled.area);
     * // => 800
     *
     * // Scale using the direct method
     * const otherScaled = bb.scale(2);
     * console.log(otherScaled.area);
     * // => 800
     * ```
     *
     * @note If you're applying the same transformation a lot of geometry,
     * creating the [[Transform]] matrix once and calling this function is faster
     * than using the direct methods.
     *
     * @category Transform
     * @param change  The [[transform]] matrix to apply to the bounding box.
     */
    transform(change: Transform): this;
}
