import { BoundingBox, Geometry, Plane, Point, PointContainment, Transform, Vector } from '../index';
/**
 * A circle is defined by a [[center]] point and a [[radius]].
 * A circle also has an orientation (defined by a [[plane]]). The orientation is important as
 * it controls where the circle's outer edge starts/ends, which is used in methods
 * like [[pointAt]], to generate points on the circle's edge.
 *
 * ### Example
 * ```ts
 * import { Circle, Point } from 'shapetypes';
 *
 * // Create a new circle
 * const circle = new Circle(10, new Point(3,4));
 *
 * // Get properties of the circle
 * console.log(circle.center.toString());
 * // => (3,4)
 * console.log(circle.area);
 * // => 314.16
 *
 * // Scale the circle
 * const larger = circle.scale(2);
 * console.log(larger.radius);
 * // => 20
 * ```
 */
export declare class Circle extends Geometry {
    /**
     * Creates a circle defined by a center point and a point on the edge. The circle
     * will be orientated so [[pointAt]] begins at the edge point.
     * @category Create
     * @param center    The center of the circle.
     * @param start     A point on the edge of the circle.
     *
     */
    static fromCenterStart(center: Point, start: Point): Circle;
    /**
     * Creates a circle that passes through the three points.
     *
     * @note  Throws an error if the three points are in a straight line.
     *
     * @category Create
     * @param p1  The first point to pass through. The circle will be orientated so
     *            [[pointAt]] begins at this point.
     * @param p2  The second point to pass through.
     * @param p3  The third point to pass through.
     */
    static fromThreePoints(p1: Point, p2: Point, p3: Point): Circle;
    private readonly _plane;
    private readonly _radius;
    /**
     * @param radius    The radius of the circle. The circle will be located at [[Point.origin]]
     *                  and aligned to [[Vector.xAxis]].
     */
    constructor(radius: number);
    /**
     * @param radius   The radius of the circle.
     * @param position The center of the circle. The circle will be aligned to [[Vector.xAxis]].
     *
     */
    constructor(radius: number, position: Point);
    /**
     * @param radius    The radius of the circle
     * @param position  The location of the circle. The cirle will be centered on the plane's origin and aligned to the plane's x-axis.
     */
    constructor(radius: number, position: Plane);
    /***
     * Gets the smallest bounding box that contains the circle.
     */
    get boundingBox(): BoundingBox;
    /**
     * Gets the center of the circle.
     */
    get center(): Point;
    /**
     * Gets the plane defining the circle's position.
     * The plane's origin is the center of the circle.
     * The plane's x-axis is the orientation of the circle.
     */
    get plane(): Plane;
    /**
     * Gets the radius of the circle.
     */
    get radius(): number;
    /**
     * Gets the diameter of the circle.
     */
    get diameter(): number;
    /**
     * Gets the circumference of the circle.
     */
    get circumference(): number;
    /**
     * Gets the area of the circle.
     */
    get area(): number;
    /***
     * Checks whether a point is inside, outside, or on the edge of a circle.
     *
     * @param testPoint   The point to test for containment.
     * @param tolerance   The distance the point can be from the edge of the circle and still considered coincident.
     */
    contains(testPoint: Point, tolerance?: number): PointContainment;
    /***
     * Finds the closest point on the circle and returns the parameter for the point.
     * @param testPoint   The target to get closest to.
     * @returns           The parameter of the closest point. Entering the parameter into [[pointAt]] will return the closest point.
     */
    closestParameter(testPoint: Point): number;
    /***
     * Finds the closest point on the circle and returns the point.
     * @param testPoint       Target to get closest to.
     * @param includeInterior If false, the closest point must lie on the outer edge of the circle.
     *                        If true, the closest point can also be a point on the interior of the circle.
     */
    closestPoint(testPoint: Point, includeInterior?: boolean): Point;
    /***
     * Checks whether another circle has the same [[plane]] and [[radius]]. Returns true if it does.
     * @param otherCircle   The circle to compare against.
     * @param tolerance     The amount the radius and plane can differ and still be considered equal.
     */
    equals(otherCircle: Circle, tolerance?: number): boolean;
    /**
     * Finds the point a certain number of radians from the start. Returns the point.
     * @param t Position of the point (in radians). This is measured counter-clockwise from the start of the circle.
     */
    pointAt(t: number): Point;
    /**
     * Finds the point a certain distance from the start. Returns the point.
     * @param distance  Position of the point. This is measured counter-clockwise from the start of the circle.
     */
    pointAtLength(distance: number): Point;
    /**
     * Finds the tangent for the circle a certain number of radians from the start.
     * @param t Position of the tangent (in radians). This is measured counter-clockwise from the start of the circle.
     */
    tangentAt(t: number): Vector;
    /***
     * Gets the circle as a string in the format: `[plane,radius]`.
     */
    toString(): string;
    /**
     * Creates a copy of the circle with a different [[area]]. The circle will be
     * in the same position but have a different radius.
     * @param newArea   The area of the new circle.
     */
    withArea(newArea: number): Circle;
    /**
     * Creates a copy of the circle with a different [[circumference]]. The circle
     * will be in the same position but have a different radius.
     * @param newCircumference    The circumference of the new circle.
     */
    withCircumference(newCircumference: number): Circle;
    /**
     * Creates a copy of the circle with a different [[diameter]]. The circle will be
     * in the same position but have a different radius/diameter.
     * @param newDiameter   The diameter of the new circle.
     */
    withDiameter(newDiameter: number): Circle;
    /**
     * Creates a copy of the circle with a different [[radius]].
     * @param newRadius   The radius of the new circle.
     */
    withRadius(newRadius: number): Circle;
    /**
     * Creates a copy of the circle with a different [[plane]]. The circle will be the
     * same size but centered on the new plane.
     * @param newPlane  The plane of the new circle.
     */
    withPlane(newPlane: Plane): Circle;
    /**
     * Creates a copy of the circle with a different [[center]]. The circle will be the
     * same size and have the same orientation but will be translated to the new center.
     * @param newCenter The center of the new circle.
     */
    withCenter(newCenter: Point): Circle;
    /***
     * Transforms the circle by a [[transform]] matrix and returns the result.
     *
     * ### Example
     * ```js
     * import { Circle, Transform } from 'shapetypes';
     *
     * // Create circle
     * const circle = new Circle(10);
     * console.log(circle.radius);
     * // => 10
     *
     * // Scale using a transform matrix
     * const matrix = Transform.scale(2);
     * const scaled = scale.transform(matrix);
     * console.log(scaled.radius);
     * // => 20
     *
     * // Scale using the direct method
     * const otherScaled = circle.scale(2);
     * console.log(otherScaled.radius);
     * // => 20
     * ```
     *
     * @note If you're applying the same transformation a lot of geometry,
     * creating the [[Transform]] matrix once and calling this function is faster
     * than using the direct methods.
     *
     * @category Transform
     * @param change  The [[transform]] matrix to apply to the cirlce.
     */
    transform(change: Transform): this;
}
