import { BoundingBox, Circle, Geometry, Point, Polygon, Polyline, Ray, Rectangle, Transform, Vector } from '../index';
/**
 * A straight edge between two points.
 *
 * ### Example
 * ```js
 * import { Line, Point, Transform, Vector } from 'shapetypes';
 *
 * // Create a new line
 * const line = new Line(new Point(1,1), new Point(4,5));
 *
 * // Get properties of the line
 * console.log(line.from.toString());
 * // => (1,1)
 * console.log(line.to.toString());
 * // => (4,5)
 * console.log(line.length);
 * // => 5
 * const mid = line.pointAt(0.5);
 * console.log(mid.toString());
 * // => (2.5,3)
 *
 * // Find closest point on the line
 * const closest = line.closestPoint(new Point(5,5));
 * console.log(closest.toString());
 * // => (4,5)
 *
 * // Translate the line with a transform matrix
 * const matrix = Transform.translate(new Vector(5, 0));
 * const moved = line.transform(matrix);
 * console.log(moved.toString());
 * // => [(6,1),(9,5)]
 */
export declare class Line extends Geometry {
    /**
     * Creates a line from a start point and a vector.
     *
     * @category Create
     * @param from      The start of the line.
     * @param direction The direction of the line.
     * @param length    The length of the line. If undefined, it will use the length of the `direction` vector.
     */
    static fromVector(from: Point, direction: Vector, length?: number): Line;
    /**
     * Creates a line from a set of coordinates.
     *
     * @category Create
     * @param coords  A 2x2 array of values specifying the line's start and end point. Format: `[[x1,y1],[x2,y2]]`.
     */
    static fromCoords(coords: readonly [readonly [number, number], readonly [number, number]]): Line;
    private readonly _from;
    private readonly _to;
    private _cacheVector;
    private _cacheBoundingBox;
    /***
     * @param from  The start of the line.
     * @param to    The end of the line.
     */
    constructor(from: Point, to: Point);
    /***
     * Gets the smallest bounding box that contains the line.
     */
    get boundingBox(): BoundingBox;
    /**
     * Gets the vector running the length of the line, starting at [[from]] and ending at [[to]].
     * The length of the vector is the length of the line.
     */
    get direction(): Vector;
    /**
     * Gets the start point of the line.
     */
    get from(): Point;
    /**
     * Gets the length of the line.
     */
    get length(): number;
    /**
     * Gets the end point of the line.
     */
    get to(): Point;
    /**
     * Gets the line's tangent vector. This vector is perpendicular to [[direction]] and is a unit vector.
     *
     * If the environment's y-axis points upwards, it will be on the left side of the line if looking [[from]] -> [[to]].
     * If the environment's y-axis points downwards, it will be on the right side of the line if looking [[from]] -> [[to]].
     */
    get unitTangent(): Vector;
    /***
     * Finds the closest point on the line and returns the parameter for the point.
     * @param testPoint                 The target to get closest to.
     * @param limitToFiniteSegment      If true, the closest point will always be within the bounds of the line. If false, the line is treated as infinite.
     * @return                          The normalized parameter of the closest point. Entering the parameter into [[pointAt]] will return the closest point.
     */
    closestParameter(testPoint: Point, limitToFiniteSegment?: boolean): number;
    /***
     * Finds the closest point on the line and returns the point.
     * @param testPoint                 The target to get closest to.
     * @param limitToFiniteSegment      If true, the closest point will always be within the bounds of the line. If false, the line is treated as infinite.
     */
    closestPoint(testPoint: Point, limitToFiniteSegment?: boolean): Point;
    /***
     * Calculates the smallest distance to a point or line.
     * @param geometry              Line or point to measure the distance to.
     * @param limitToFiniteSegment  If false, the line is treated as infinite.
     */
    distanceTo(geometry: Point | Line, limitToFiniteSegment?: boolean): number;
    /***
     * Checks whether another line has the same start and end points. Returns true if it does.
     * @param otherLine   The line to compare against.
     * @param tolerance   The amount that the points can differ and still be considered equal.
     */
    equals(otherLine: Line, tolerance?: number): boolean;
    /**
     * Lengthens the line by extending the end points. Returns the resulting line.
     * @param fromDistance  The distance to move [[from]]. If 0, [[from]] will remain in place. If greater than 0, the line will lengthen.
     * @param toDistance    The distance to move [[to]]. If 0, [[to]] will remain in place. If greater than 0, the line will lengthen.
     */
    extend(fromDistance: number, toDistance: number): Line;
    /**
     * Creates a copy of the line with [[from]] and [[to]] swapped.
     */
    flip(): Line;
    /***
     * Calculates where the line intersects other geometry and returns the parameters
     * for these points of intersection.
     *
     * @note              This is an alias for the [[Intersection.line]] function.
     * @note              Only accounts for crossings, not coincident overlaps.
     *
     * @param otherGeom   The geometry to intersect.
     * @returns           The parameters of the intersection points.
     *                    The array is always sorted from the smallest to largest parameter.
     *                    Entering the parameter into [[pointAt]] will return the intersection point.
     */
    intersection(otherGeom: Point | Line | Ray | BoundingBox | Circle | Rectangle | Polyline | Polygon | ReadonlyArray<Point | Line | Ray | BoundingBox | Circle | Rectangle | Polyline | Polygon>): readonly number[];
    /**
     * Finds the point a normalized parameter along the line. Returns the point.
     *
     * The parameter ranges from 0, which is the line's start ([[from]]),
     * through to 1, which is the line's end ([[to]]).
     * The mid point of the line is 0.5.
     *
     * ### Example
     * ```js
     * let line = new Line(new Point(0, 0), new Point(10, 0));
     * console.log(line.pointAt(0).toString());
     * // => (0,0)
     * console.log(line.pointAt(0.5).toString());
     * // => (5,0)
     * console.log(line.pointAt(1).toString());
     * // => (10,0)
     * ```
     * @param u                     The normalized parameter.
     * @param limitToFiniteSegment  If true, the point will always be within the bounds of the line (u-values less than 0 will become 0 and values greater than 1 will become 1). If false, the line is treated as infinite.
     */
    pointAt(u: number, limitToFiniteSegment?: boolean): Point;
    /**
     * Finds the point a given distance from the line's start ([[from]]) and returns the point.
     * @param distance              The distance from the [[from]] point.
     * @param limitToFiniteSegment  If true, the point will always be within the bounds of the line. If false, the line is treated as infinite.
     */
    pointAtLength(distance: number, limitToFiniteSegment?: boolean): Point;
    /***
     * Gets the line as a string in the format: `[(x,y),(x,y)]`.
     */
    toString(): string;
    /**
     * Creates a copy of the line with a different [[from]] point.
     * @param newFrom
     */
    withFrom(newFrom: Point): Line;
    /**
     * Creates a copy of the line with a different length. The [[from]] point will remain
     * fixed in place and the [[to]] point will be moved to make the line the right length.
     * @param distance  New length for the line. If the length is set to a negative number, the line's [[direction]] will be reversed but the length will remain positive.
     */
    withLength(distance: number): Line;
    /**
     * Creates a copy of the line with a different [[to]] point.
     * @param newTo
     */
    withTo(newTo: Point): Line;
    /***
     * Transforms the line by a [[transform]] matrix and returns the result.
     *
     * ### Example
     * ```js
     * import { Line, Point, Transform, Vector } from 'shapetypes';
     *
     * // Create a line
     * const line = new Line(new Point(0,0), new Point(10,0));
     * console.log(line.toString());
     * // => [(0,0),(10,0)]
     *
     * // Translate using a transform matrix
     * const matrix = Transform.translate(new Vector(5, 4));
     * const moved = line.transform(matrix);
     * console.log(moved.toString());
     * // => [(5,4),(15,4)]
     *
     * // Translate using the direct method
     * const otherMoved = line.translate(5, 4);
     * console.log(otherMoved.toString());
     * // => [(5,4),(15,4)]
     * ```
     *
     * @note If you're applying the same transformation a lot of geometry,
     * creating the [[Transform]] matrix once and calling this function is faster
     * than using the direct methods.
     *
     * @category Transform
     * @param change  The [[transform]] matrix to apply to the line.
     */
    transform(change: Transform): this;
}
