import { BoundingBox, Circle, Geometry, Line, Point, Polygon, Polyline, Rectangle, Transform, Vector } from '../index';
/**
 * The extent of a ray.
 * A ray can be limited to only shooting forward (either [[positive]] or [[positiveAndZero]]),
 * or the ray can shoot both forward and backward ([[both]]).
 */
export declare enum RayRange {
    /** The ray shoots in one direction from the start point to infinity but doesn't include the start point.  */
    positive = 0,
    /** The ray shoots in one direction from the start point to infinity and includes the start point.  */
    positiveAndZero = 1,
    /** The ray shoots in two directions: from the start point to infinity, and in the opposite direction to negative infinity.  */
    both = 2
}
/**
 * Checks whether the value is in the range specified. Returns true if it is.
 * @ignore
 */
export declare function inRayRange(distance: number, range: RayRange): boolean;
/**
 * A line of infinite length. A ray has a start point ([[from]]) and a direction ([[direction]]) but no end point.
 *
 * In general, the ray is considered infinite in both directions, meaning it shoots
 * both forward and back. Certain methods accept [[RayRange]] as a parameter,
 * which allows you to specify whether the ray should shoot in both directions or only forward.
 *
 * ### Example
 * ```js
 * import { Point, Ray, Vector } from 'shapetypes';
 *
 * // Create a new ray
 * const ray = new Ray(new Point(3,4), new Vector(1,0));
 *
 * // Get properties of the ray
 * console.log(ray.from.toString());
 * // => (3,4)
 * console.log(ray.direction.toString());
 * // => ⟨1,0⟩
 *
 * // Get a point on the ray
 * const p = ray.pointAt(5);
 * console.log(p.toString());
 * // => (8,4)
 *
 * // Get parameter of intersection between point and ray
 * console.log(ray.intersection(p));
 * // => [5]
 *
 * // Translate the ray
 * const transformed = ray.translate(new Vector(3,4));
 * console.log(transformed.from.toString());
 * // => (6,8)
 * ```
 */
export declare class Ray extends Geometry {
    /**
     * Creates a new ray from two points.
     * @category Create
     * @param from        The start of the ray.
     * @param pointOnRay  A point on the ray. The [[direction]] of the ray will start at `from` and point towards `pointOnRay`.
     */
    static fromPoints(from: Point, pointOnRay: Point): Ray;
    private readonly _from;
    private readonly _direction;
    /***
     * @param from        The start of the ray.
     * @param direction   The direction the ray shoots.
     */
    constructor(from: Point, direction: Vector);
    /**
     * Gets the start point of the ray.
     */
    get from(): Point;
    /**
     * Gets the direction that the ray shoots. Is always a unit vector.
     */
    get direction(): Vector;
    /***
     * Finds the closest point on the ray and returns the parameter for the point.
     * @param testPoint     The target to get closest to.
     * @param range         The extent of the ray. Specifies whether the ray is
     *                      shooting both forward and backward, or only forward.
     * @returns             The parameter of the closest point. The parameter is the
     *                      distance between [[from]] and the closest point.
     *                      If the value is negative, it means the closest point is in the
     *                      opposite direction to the [[direction]] vector.
     *                      Entering the parameter into [[pointAt]] will return the closest point.
     */
    closestParameter(testPoint: Point, range?: RayRange): number;
    /***
     * Finds the closest point on the ray and returns the point.
     * @param testPoint         The target to get closest to.
     * @param range             The extent of the ray. Specifies whether the ray is
     *                          shooting both forward and backward, or only forward.
     */
    closestPoint(testPoint: Point, range?: RayRange): Point;
    /***
     * Checks whether another ray has the same [[from]] point and [[direction]]. Returns true if it does.
     * @param otherRay    The ray to compare against.
     * @param tolerance   The amount the point and vector can differ and still be considered equal.
     */
    equals(otherRay: Ray, tolerance?: number): boolean;
    /***
     * Calculates where the ray intersects other geometry and returns the parameters
     * for these points of intersection.
     *
     * @note              This is an alias for the [[Intersection.ray]] function.
     * @note              Only accounts for crossings, not coincident overlaps.
     *
     * @param otherGeom   The geometry to intersect.
     * @param range       The extent of the ray. Specifies whether the ray is
     *                    shooting both forward and backward, or only forward.
     * @returns           The parameters of the intersection points.
     *                    The array is always sorted from the smallest to largest parameter.
     *                    The parameter is the distance between [[from]] and the intersection point.
     *                    If the value is negative, it means the point of intersection is in the
     *                    opposite direction to the [[direction]] vector.
     *                    Entering the parameter into [[pointAt]] will return the intersection point.
     */
    intersection(otherGeom: Point | Line | Ray | BoundingBox | Circle | Rectangle | Polyline | Polygon | ReadonlyArray<Point | Line | Ray | BoundingBox | Circle | Rectangle | Polyline | Polygon>, range?: RayRange): readonly number[];
    /**
     * Finds the point a given distance from the ray's start ([[from]]). Returns the point.
     * @param distance  The distance from the [[from]] point.
     *                  If positive, it will be in the direction of [[direction]].
     *                  If negative, it will be in the inverse.
     */
    pointAt(distance: number): Point;
    /***
     * Gets the ray as a string in the format: `[from,direction]`.
     */
    toString(): string;
    /**
     * Creates a copy of the ray with a different [[from]] point.
     * @param newFrom   The new location of the [[from]] point.
     */
    withFrom(newFrom: Point): Ray;
    /**
     * Creates a copy of the ray with a different [[direction]] vector.
     * @param newDirection    The new [[direction]] of the ray.
     */
    withDirection(newDirection: Vector): Ray;
    /***
     * Transforms the ray by a [[Transform]] matrix and returns the result.
     *
     * ### Example
     * ```js
     * import { Point, Ray, Transform, Vector } from 'shapetypes';
     *
     * // Create a new ray
     * const ray = new Ray(new Point(2,2), Vector.worldX());
     * console.log(ray.from.toString());
     * // => (2,2)
     *
     * // Translate ray using a transform matrix
     * const matrix = Transform.translate(new Vector(3,4));
     * const moved = ray.transform(matrix);
     * console.log(moved.from.toString());
     * // => (5,6)
     *
     * // Translate ray using the direct method
     * const otherMoved = ray.translate(new Vector(3,4));
     * console.log(otherMoved.from.toString());
     * // => (5,6)
     * ```
     *
     * @note  If you're applying the same transformation to a lot of geometry,
     * creating the [[Transform]] matrix once and calling this function is faster
     * than using the direct methods.
     *
     * @category Transform
     * @param change  The [[transform]] matrix to apply to the ray.
     */
    transform(change: Transform): this;
}
