import { Geometry, Point, Transform } from '../index';
/**
 * A two-dimensional vector. The vector is defined by an [[x]] magnitude and a [[y]] magnitude.
 *
 * ### Example
 * ```js
 * import { Vector } from 'shapetypes';
 *
 * // Create a new vector
 * const v = new Vector(10, 0);
 *
 * // Get properties of the vector
 * console.log(v.x);
 * // => 10
 * console.log(v.length);
 * // => 10
 *
 * // Measure the angle between two vectors
 * const other = new Vector(0,1);
 * const angle = v.angle(other);
 * console.log(angle);
 * // => 1.57
 *
 * // Rotate a vector
 * const rotated = v.rotate(Math.PI / 2);
 * console.log(rotated.toString());
 * // => `⟨0,10⟩`
 * ```
 */
export declare class Vector extends Geometry {
    /**
     * Creates a new vector from two points. The vector starts at `from` and ends at `to`.
     *
     * @category Create
     * @param from    Start of the vector.
     * @param to      End of the vector.
     */
    static fromPoints(from: Point, to: Point): Vector;
    /**
     * Returns the world's x-axis as a unit vector. Equivalent to `new Vector(1, 0)`.
     *
     * @category Create
     */
    static worldX(): Vector;
    /**
     * Returns the world's y-axis as a unit vector. Equivalent to `new Vector(0, 1)`.
     *
     * @category Create
     */
    static worldY(): Vector;
    private static readonly _worldX;
    private static readonly _worldY;
    private readonly _x;
    private readonly _y;
    private _cacheLength;
    /***
     * @param x   Magnitude in the x-direction.
     * @param y   Magnitude in the y-direction.
     */
    constructor(x: number, y: number);
    /**
     * Checks whether the vector's length is 1. Returns true if it is.
     */
    get isUnit(): boolean;
    /**
     * Checks whether the vector's length is 0. Returns true if it is.
     */
    get isZero(): boolean;
    /**
     * Gets the vector's length/magnitude.
     */
    get length(): number;
    /**
     * Gets the vector's magnitude in the x-direction.
     */
    get x(): number;
    /**
     * Splits the vector into its x & y components and returns the x component.
     *
     * This is the same as [[worldX]] * [[x]].
     */
    get xAxis(): Vector;
    /**
     * Gets the vector's magnitude in the y-direction.
     */
    get y(): number;
    /**
     * Splits the vector into its x & y components and returns the y component.
     *
     * This is the same as [[worldY]] * [[y]].
     */
    get yAxis(): Vector;
    /***
     * Adds the x & y magnitudes of another vector and returns the resulting vector.
     * @param addend  Vector to add.
     */
    add(addend: Vector): Vector;
    /**
     * Adds an x & y value to the vector and returns the resulting vector.
     * @param x       Value to add to the x magnitude.
     * @param y       Value to add to the y magnitude.
     */
    add(x: number, y: number): Vector;
    /**
     * Calculates the smallest angle between this vector and another vector.
     * The result is measured in radians and will be between 0 and Math.PI.
     *
     * @param other   The vector to measure angle between.
     * @returns       The smallest angle between the two vectors in radians.
     */
    angle(other: Vector): number;
    /**
     * Calculates the signed angle between this vector and another vector.
     * The result is measured in radians.
     *
     * A positive value means that `other` is counter-clockwise from the vector
     * (assuming the environment's y-axis points upwards, otherwise it's clockwise).
     *
     * A negative value means that `other` is clockwise from the vector
     * (assuming the environment's y-axis points upwards, otherwise it's counter-clockwise).
     *
     * @param other   The vector to measure the angle between.
     * @returns       The signed angle in radians.
     */
    angleSigned(other: Vector): number;
    /***
     * Divides the x & y magnitudes by a value and returns the resulting vector.
     *
     * @param denominator Amount to divide the magnitudes by.
     */
    divide(denominator: number): Vector;
    /**
     * Divides the x & y magnitudes by different amounts in the x- and y-direction. Returns the resulting vector.
     * @param denominatorX    Amount to divide the x magnitude by.
     * @param denominatorY    Amount to divide the y magnitude by.
     */
    divide(denominatorX: number, denominatorY: number): Vector;
    /**
     * Calculates the dot product of this vector and another vector. Returns the resulting vector.
     * @param other Vector to calculate dot product with.
     */
    dotProduct(other: Vector): number;
    /***
     * Checks whether another vector has the same [[x]] and [[y]] values. Returns true if it does.
     * @param otherVector   The vector to compare against.
     * @param tolerance     The amount that the [[x]] and [[y]] values of the
     *                      vectors can differ and still be considered equal.
     */
    equals(otherVector: Vector, tolerance?: number): boolean;
    /**
     * Checks whether another vector is parallel. Returns true if it is.
     * @param test              Vector to compare against.
     * @param angleTolerance    Acceptable deviation from parallel, measured as the angle between vectors (in radians).
     */
    isParallelTo(test: Vector, angleTolerance?: number): boolean;
    /**
     * Checks whether another vector is perpendicular. Returns true if it is.
     * @param test              Vector to compare against.
     * @param angleTolerance    Acceptable deviation from perpendicular, measured as the angle between vectors (in radians).
     */
    isPerpendicularTo(test: Vector, angleTolerance?: number): boolean;
    /**
     * Checks whether the vector is parallel to [[worldX]]. Returns true if it is.
     */
    isXAxis(): boolean;
    /**
     * Checks whether the vector is parallel to [[worldY]]. Returns true if it is.
     */
    isYAxis(): boolean;
    /***
     * Multiplies the x & y magnitudes by a value and returns the resulting vector.
     *
     * @param factor  Amount to multiply the magnitudes by.
     */
    multiply(factor: number): Vector;
    /**
     * Multiplies the x & y magnitudes by different amounts in the x- and y-direction. Returns the resulting vector.
     * @param factorX   Amount to multiply the x magnitude by.
     * @param factorY   Amount to multiply the y magnitude by.
     */
    multiply(factorX: number, factorY: number): Vector;
    /**
     * Creates a new vector that is perpendicular to this vector.
     *
     * The new vector will be on the left side of this vector if looking along [[from]]->[[to]]
     * (assuming the environment's y-axis points upwards, if it doesn't it will be on the right side).
     */
    perpendicular(): Vector;
    /**
     * Inverts the direction of the vector and returns the inverted vector. This is the same as
     * multiplying the vector by -1.
     */
    reverse(): Vector;
    /***
     * Subtracts the x & y magnitudes of another vector and returns the resulting vector.
     * @param subtrahend Vector to subtract.
     */
    subtract(subtrahend: Vector): Vector;
    /**
     * Subtracts an x & y value from the vector and returns the resulting vector.
     * @param x       Value to subtract from the x magnitude.
     * @param y       Value to subtract from the y magnitude.
     */
    subtract(x: number, y: number): Vector;
    /***
     * Gets the vector as a string in the format: '⟨x,y⟩'.
     */
    toString(): string;
    /**
     * Creates a copy of the vector where the length is equal to 1.
     */
    unitize(): Vector;
    /**
     * Creates a copy of the vector with the x and y magnitudes scaled to a given length.
     * @param newLength Length of new vector. If negative, the vector will be inverted but the resulting length will be positive.
     */
    withLength(newLength: number): Vector;
    /**
     * Creates a copy of the vector with a different x magnitude.
     * @param newX New value for x.
     */
    withX(newX: number): Vector;
    /**
     * Creates a copy of the vector with a different y magnitude.
     * @param newY New value for y.
     */
    withY(newY: number): Vector;
    /***
     * Transforms the vector by a [[Transform]] matrix and returns the result.
     *
     * ### Example
     * ```js
     * import { Transform, Vector } from 'shapetypes';
     *
     * // Create a new vecctor
     * const vector = new Vector(3, 4);
     * console.log(vector.length);
     * // => 5
     *
     * // Scale vector using a transform matrix
     * const matrix = Transform.scale(2);
     * const scaled = vector.transform(matrix);
     * console.log(vector.length);
     * // => 10
     *
     * // Scale vector using the direct method
     * const otherScaled = vector.scale(2);
     * console.log(otherScaled.length);
     * // => 10
     * ```
     *
     * @note If you're applying the same transformation a lot of geometry,
     * creating the [[Transform]] matrix once and calling this function is faster
     * than using the direct methods.
     *
     * @category Transform
     * @param change  The [[transform]] matrix to apply to the vector.
     */
    transform(change: Transform): this;
}
